// Begin $Header: /home/uda/src/c/buduscript/RCS/plugin.h,v 1.38 2011/12/07 19:25:57 uda Exp $
// UDA'Software 2003
// $Date: 2011/12/07 19:25:57 $
// $Revision: 1.38 $

/*
 * PLUGIN.H : Questa classe ha il compito di gestire l'interfaccia
 * verso il client XChat2, offrendo tutti gli oggetti necessari a
 * facilitare l'uso delle varie funzioni offerte dalla libreria base
 * scritta in C Ansi.
*/

/*

  Copyright (C) 1988 Free Software Foundation, Inc.

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 1, or (at your option)
  any later version.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
#ifndef HEADER_PLUGIN
#define HEADER_PLUGIN

// Include la libreria per la gestione dei plugin.
#include <src/common/xchat-plugin.h>

// Include le librerie standard
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <ctype.h>

// Include la libreria GLIB
#include <glib.h>

#ifdef DMALLOC
#include "dmalloc.h"
#endif

#define	DONT_FREE_PREVIOUS_LIST	1

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
#define IGNORED_NO		0
#define IGNORED_YES		1
/*
 * Definisce una classe User che serve per manipolare gli utenti
 * quando si effettuano delle ricerche all'interno del database del
 * client attraverso l'interfaccia della classe Plugin.
 *
 */
class UserData {
  char		*psNickname;
  char		*psHost;
  char		*psPrefix;
  char		*psChannel;
  char		*psServer;
  xchat_context	*pCTX;
  int		bIgnored;
  
 public:
  // Il costruttore
  UserData(const char		*Nickname = NULL, 
	   const char		*Host = NULL, 
	   const char		*Prefix = NULL, 
	   const char		*Channel = NULL, 
	   const char		*Server = NULL, 
	   xchat_context	*CTX = NULL, 
	   int			Ignored = IGNORED_NO) :
    pCTX(CTX), 
    bIgnored(Ignored)
    {
      if (Nickname)
	psNickname = strdup(Nickname);
      
      if (Host)
	psHost = strdup(Host);
      
      if (Prefix)
	psPrefix = strdup(Prefix);
      
      if (Channel)
	psChannel = strdup(Channel);
      
      if (Server)
	psServer = strdup(Server);
    }

  // Il costruttore di copia.
  UserData(const UserData &user) {
    psNickname = strdup(user.psNickname);
    psHost = strdup(user.psHost);
    psPrefix = strdup(user.psPrefix);
    psChannel = strdup(user.psChannel);
    psServer = strdup(user.psServer);
    pCTX = user.pCTX;
    bIgnored = user.bIgnored;
  }

  // Il distruttore
  ~UserData() {
    free(psNickname); psNickname = NULL;
    free(psHost); psHost = NULL;
    free(psPrefix); psPrefix = NULL;
    free(psChannel); psChannel = NULL;
    free(psServer); psServer = NULL;
    pCTX = NULL;
    bIgnored = 0;
  }
  
  // Funzioni che ritornano i dati.
  inline const char*	GetNickname()	{return psNickname;}
  inline const char*	GetHost()	{return psHost;}
  inline const char*	GetPrefix()	{return psPrefix;}
  inline const char*	GetChannel()	{return psChannel;}
  inline const char*	GetServer()	{return psServer;}
  inline xchat_context*	GetCTX()	{return pCTX;}
  inline int		GetIgnored()	{return bIgnored;}

  // Funzione che imposta il flag dell'ignore.
  int SetIgnored(const char *mask);

  // Overloading dell'operatore == .
  int operator == (const UserData user) {
    if (psNickname && user.psNickname && !strcasecmp(psNickname, user.psNickname)) {
      if (psHost && user.psHost && !strcasecmp(psHost, user.psHost)) {
	if (psPrefix && user.psPrefix && !strcasecmp(psPrefix, user.psPrefix)) {
	  if (psChannel && user.psChannel && !strcasecmp(psChannel, user.psChannel)) {
	    if (psServer && user.psServer && !strcasecmp(psServer, user.psServer))
	      return 1;
	  }
	}
      }
    }

    return 0;
  }
    
  // Overloading dell'operatore != .
  int operator != (const UserData user) {
    if (*this == user) {
      return 0;
    }
    
    // Ritorna fallimento.
    return 1;
  }
    
  // Overloading dell'operatore < .
  int operator < (const UserData user) {
    if (psNickname && user.psNickname && strcasecmp(psNickname, user.psNickname) < 0) {
      return 1;
    }
    
    // Ritorna fallimento.
    return 0;
  }
    
  // Overloading dell'operatore <=.
  int operator <= (const UserData user) {
    if (psNickname && user.psNickname && strcasecmp(psNickname, user.psNickname) <= 0) {
      return 1;
    }
    
    // Ritorna fallimento.
    return 0;
  }
    
  // Overloading dell'operatore > .
  int operator > (const UserData user) {
    if (psNickname && user.psNickname && strcasecmp(psNickname, user.psNickname) > 0) {
      return 1;
    }
    
    // Ritorna fallimento.
    return 0;
  }
    
  // Overloading dell'operatore >= .
  int operator >= (const UserData user) {
    if (psNickname && user.psNickname && strcasecmp(psNickname, user.psNickname) >= 0) {
      return 1;
    }
    
    // Ritorna fallimento.
    return 0;
  }
};

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
typedef int (xchat_cmd_cb)	(char *word[], char *word_eol[], void *user_data);
typedef int (xchat_serv_cb)	(char *word[], char *word_eol[], void *user_data);
typedef int (xchat_print_cb)	(char *word[], void *user_data);
typedef int (xchat_fd_cb)	(int fd, int flags, void *user_data);
typedef int (xchat_timer_cb)	(void *user_data);

/*
 * Deve dichiarare delle strutture dati e delle enumerazioni
 * appartenenti al modulo plugin.c di XChat 2 perche' non sono
 * altrimenti accessibili.
 *
 */
struct _xchat_list {
  int type;		/* LIST_* */
  GSList *pos;		/* current pos */
  GSList *next;		/* next pos */
  GSList *head;
};

enum {
  LIST_CHANNELS,
  LIST_DCC,
  LIST_IGNORE,
  LIST_USERS
};

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Questa funzione ha il compito di convertire i comandi %C, %B, %U,
 * %O e %R nei rispettivi codici.
 *
 */
void
check_special_chars (char *cmd, int do_ascii) {
  int	occur = 0;
  int	len = strlen (cmd);
  char	*buf, *utf;
  char	tbuf[4];
  int	i = 0, j = 0;
  gsize	utf_len;

  if (!len)
    return;

  buf = (char*)malloc(len + 1);

  if (buf) {
      while (cmd[j]) {
	  switch (cmd[j]) {
	  case '%':
	    occur++;
	    if (do_ascii &&
		j + 3 < len &&
		(isdigit (cmd[j + 1]) && isdigit (cmd[j + 2]) &&
		 isdigit (cmd[j + 3]))) {
	      tbuf[0] = cmd[j + 1];
	      tbuf[1] = cmd[j + 2];
	      tbuf[2] = cmd[j + 3];
	      tbuf[3] = 0;
	      buf[i] = atoi (tbuf);
	      utf = g_locale_to_utf8 (buf + i, 1, 0, &utf_len, 0);
	      if (utf) {
		memcpy (buf + i, utf, utf_len);
		g_free (utf);
		i += (utf_len - 1);
	      }
	      j += 3;
	    } else {
	      switch (cmd[j + 1]) {
	      case 'R':
		buf[i] = '\026';
		break;
	      case 'U':
		buf[i] = '\037';
		break;
	      case 'B':
		buf[i] = '\002';
		break;
	      case 'C':
		buf[i] = '\003';
		break;
	      case 'O':
		buf[i] = '\017';
		break;
	      case '%':
		buf[i] = '%';
		break;
	      default:
		buf[i] = '%';
		j--;
		break;
	      }
	      j++;
	      break;
	    default:
	      buf[i] = cmd[j];
	    }
	  }
	  j++;
	  i++;
      }
      buf[i] = 0;
      if (occur)
	strncpy (cmd, buf, len);
      
      free(buf);
  }
}
 
// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Queste funzioni servono per convertire in minuscolo un carattere
 * (rispettando l'RFC) e per verificare se una stringa corrisponde ad
 * una data maschera (controllo dell'ignore).
 *
 */
 
#define rfc_tolower(c)		(rfc_tolowertab[(unsigned char)(c)])

const unsigned char rfc_tolowertab[] = {
  0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
  0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
  0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
  0x1e, 0x1f,
  ' ', '!', '"', '#', '$', '%', '&', 0x27, '(', ')',
  '*', '+', ',', '-', '.', '/',
  '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
  ':', ';', '<', '=', '>', '?',
  '@', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
  'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
  't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~',
  '_',
  '`', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i',
  'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's',
  't', 'u', 'v', 'w', 'x', 'y', 'z', '{', '|', '}', '~',
  0x7f,
  0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86, 0x87, 0x88, 0x89,
  0x8a, 0x8b, 0x8c, 0x8d, 0x8e, 0x8f,
  0x90, 0x91, 0x92, 0x93, 0x94, 0x95, 0x96, 0x97, 0x98, 0x99,
  0x9a, 0x9b, 0x9c, 0x9d, 0x9e, 0x9f,
  0xa0, 0xa1, 0xa2, 0xa3, 0xa4, 0xa5, 0xa6, 0xa7, 0xa8, 0xa9,
  0xaa, 0xab, 0xac, 0xad, 0xae, 0xaf,
  0xb0, 0xb1, 0xb2, 0xb3, 0xb4, 0xb5, 0xb6, 0xb7, 0xb8, 0xb9,
  0xba, 0xbb, 0xbc, 0xbd, 0xbe, 0xbf,
  0xc0, 0xc1, 0xc2, 0xc3, 0xc4, 0xc5, 0xc6, 0xc7, 0xc8, 0xc9,
  0xca, 0xcb, 0xcc, 0xcd, 0xce, 0xcf,
  0xd0, 0xd1, 0xd2, 0xd3, 0xd4, 0xd5, 0xd6, 0xd7, 0xd8, 0xd9,
  0xda, 0xdb, 0xdc, 0xdd, 0xde, 0xdf,
  0xe0, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5, 0xe6, 0xe7, 0xe8, 0xe9,
  0xea, 0xeb, 0xec, 0xed, 0xee, 0xef,
  0xf0, 0xf1, 0xf2, 0xf3, 0xf4, 0xf5, 0xf6, 0xf7, 0xf8, 0xf9,
  0xfa, 0xfb, 0xfc, 0xfd, 0xfe, 0xff
};

int match_mask(const char *mask,	// La maschera
	       const char *string) {	// La stringa
  register const char	*m = mask, *s = string;
  register char		ch;
  const char		*bm, *bs;

  while ((ch = *m++) && (ch != '*'))
    switch (ch) {
    case '\\':
      if (*m == '?' || *m == '*')
	ch = *m++;
    default:
      if (rfc_tolower(*s) != rfc_tolower(ch))
	return 0;
    case '?':
      if (!*s++)
	return 0;
    };

  if (!ch)
    return !(*s);

 got_star:
  bm = m;
  while ((ch = *m++))
    switch (ch) {
    case '?':
      if (!*s++)
	return 0;
    case '*':
      bm = m;
      continue;
    case '\\':
      if (*m == '?' || *m == '*')
	ch = *m++;

    default:
      goto break_while;
    };

 break_while:
  if (!ch)
    return 1;

  ch = rfc_tolower(ch);
  while (rfc_tolower(*s++) != ch)
    if (!*s)
      return 0;
  
  bs = s;

  while ((ch = *m++)) {
    switch (ch) {
    case '*':
      goto got_star;
    case '\\':
      if (*m == '?' || *m == '*')
	ch = *m++;
      
    default:
      if (rfc_tolower(*s) != rfc_tolower(ch)) {
	m = bm;
	s = bs;
	
	goto got_star;
      };
    case '?':
      if (!*s++)
	return 0;
    };
  };

  if (*s) {
    m = bm;
    s = bs;
    goto got_star;
  };

  return 1;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
/*
 * Definisce la funzione che imposta il flag dell'ignore nella classe
 * UserData.
 *
 * (lo fa dopo la definizione della funzione match_mask() necessaria
 * allo scopo).
 *
 */
int
UserData::SetIgnored(const char *mask) {
  if (mask &&		// La maschera deve essere valida
      mask[0] != '\0' && 
      psNickname &&	// Il nickname deve essere valido
      psHost) {		// L'host deve essere valido
    char	*string = (char*)malloc(strlen(psNickname) + strlen(psHost) + 2);

    sprintf(string, "%s!%s", psNickname, psHost);
    bIgnored = match_mask(mask, string);
    
    free(string); string = NULL;

    return bIgnored;
  }

  return -1;
}

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
class Plugin {
  xchat_plugin	*hPlugin;		// Handle del plugin
  xchat_list	*hlData;		// Lista dei dati
  char		*psNome;		// Nome del plugin
  char		*psDescrizione;		// Descrizione del plugin
  char		*psVersione;		// Versione del plugin
  char		*psArgomenti;		// Argomenti passati al plugin
  int		eListStatus;		// Stato nella gestione delle liste

 public:
  enum {list_ERROR, list_OK};		// Enumerazione sullo stato della lista

  // Costruttore
  Plugin(xchat_plugin *hP = NULL,
	 const char *Nome = "",
	 const char *Descrizione = "",
	 const char *Versione = "",
	 const char *Argomenti = "") :
    hPlugin(hP), 
    hlData(NULL)
    {
      psNome = NULL;
      psDescrizione = NULL;
      psVersione = NULL;
      psArgomenti = NULL;
      
      if (Nome)
	psNome = strdup(Nome);
      
      if (Descrizione)
	psDescrizione = strdup(Descrizione);
      
      if (Versione)
	psVersione = strdup(Versione);

      if (Argomenti)
	psArgomenti = strdup(Argomenti);

      eListStatus = list_ERROR;
    }
  
  // Costruttore di Copia (necessario per una classe con puntatori)
  Plugin(const Plugin &plugin) {
    hPlugin = plugin.hPlugin;
    hlData = plugin.hlData;

      psNome = NULL;
      psDescrizione = NULL;
      psVersione = NULL;
      psArgomenti = NULL;
      
      if (plugin.psNome)
	psNome = strdup(plugin.psNome);
      
      if (plugin.psDescrizione)
	psDescrizione = strdup(plugin.psDescrizione);
      
      if (plugin.psVersione)
	psVersione = strdup(plugin.psVersione);

      if (plugin.psArgomenti)
	psArgomenti = strdup(plugin.psArgomenti);

      eListStatus = plugin.eListStatus;
  }
  
  // Ritorna il puntatore al plugin.
  xchat_plugin	*GetPlugin() {
    return hPlugin;
  }

  // Ritorna il Nome del plugin.
  const char	*GetNome() {
    return psNome;
  }

  // Ritorna la Versione.
  const char	*GetVersione() {
    return psVersione;
  }

  // Ritorna la lista dei dati.
  xchat_list	*GetList() {
    if (eListStatus == list_OK)
      return hlData;
    else
      return 0;
  }

  // Ritorna lo stato della lista dati.
  int		GetListStatus() {
    return eListStatus;
  }

  // Aggiunge un nuovo comando.
  xchat_hook	*hook_command(const char *name,			// Nome del comando
			      xchat_cmd_cb *callb,		// Funzione da invocare
			      const char *help_text = NULL,	// Guida
			      void *userdata = NULL) {		// Puntatore passato alla funzione
    // Verifica che i dati passati siano corretti.
    if (name && name[0] != '\0') {
      return xchat_hook_command(hPlugin, 
				name, 
				XCHAT_PRI_NORM, 
				callb, 
				help_text, 
				userdata);
    }

    return NULL;
  }

  // Aggancia un Socket o un Descrittore file.
  xchat_hook	*hook_fd(int fd,				// Il File Descriptor
			 // Flag: XCHAT_FD_READ, XCHAT_FD_WRITE, XCHAT_FD_EXCEPTION
			 int flags, 
			 xchat_fd_cb *callb,			// Funzione da invocare
			 void *userdata = NULL) {		// Puntatore passato alla funzione
    return xchat_hook_fd(hPlugin, 
			 fd, 
			 flags, 
			 callb, 
			 userdata);
  }

  // Aggancia un evento tra quelli dell'Event Text di XChat o tra
  // quelli speciali.
  /*
   * Lista Eventi Speciali:
   * "Open Context" - Chiamata quando un nuovo xchat_context e' creato.
   * "Close Context" - Chiamata quando un puntatore xchat_context e' chiuso.
   * "Focus Tab" - Chiamata quando una Tab riceve il focus.
   * "Focus Window" - Chiamata quando la finestra principale riceve il focus.
   * "DCC Chat Text" - Chiamata quando si riceve un DCC. L'array
   * word[] viene cosi' riempito:
   * word[1] Address 
   * word[2] Port 
   * word[3] Nick 
   * word[4] The Message
   */
  xchat_hook	*hook_print(const char *name,			// Nome dell'Event Text
			    xchat_print_cb *callb,		// Funzione da invocare
			    void *userdata = NULL) {		// Puntatore passato alla funzione
    // Controlla che il parametro passato sia valido.
    if (name && name[0] != '\0') {
      return xchat_hook_print(hPlugin, 
			      name, 
			      XCHAT_PRI_NORM, 
			      callb, 
			      userdata);
    }

    return NULL;
  }

  // Aggancia un Evento del Server.
  xchat_hook	*hook_server(const char *name,			// Nome dell'evento
			     xchat_serv_cb *callb,		// Funziona da invocare
			     void *userdata = NULL) {		// Puntatore passato alla funzione
    // Controlla che il parametro passato sia valido.
    if (name && name[0] != '\0') {
      return xchat_hook_server(hPlugin, 
			       name, 
			       XCHAT_PRI_NORM, 
			       callb, 
			       userdata);
    }

    return NULL;
  }

  // Attiva un Timer.
  xchat_hook	*hook_timer(int timeout,			// Tempo del timer in millisecondi
			    xchat_timer_cb *callb,		// Funzione da invocare
			    void *userdata = NULL) {		// Puntatore passato alla funzione
    if (timeout > 0)
      return xchat_hook_timer(hPlugin, 
			      timeout, 
			      callb, 
			      userdata);

    return NULL;
  }

  // Disattiva un evento agganciato.
  void		unhook(xchat_hook **hook) {			// Puntatore all'Hook da eliminare
    if (hook) {
      xchat_unhook(hPlugin, (*hook));
      (*hook) = NULL;
    }
  }

  // Esegue un comando XChat.
  void		command(char *command) {
    if (command && command[0] != '\0') {
      check_special_chars(command, 1);
      xchat_command(hPlugin, command);
    }
  }

  // Esegue un comando XChat composto.
  void		commandf(const char *format, ...) {
    va_list args;
    char *buf;
    
    va_start(args, format);
    buf = g_strdup_vprintf(format, args);
    va_end(args);
    
    check_special_chars(buf, 1);
    xchat_command(hPlugin, buf);
    g_free(buf);
  }

  // Stampa una stringa.
  void		print(const char *text) {
    if (text && text[0] != '\0') {
      xchat_print(hPlugin, text);
    }
  }

  // Stampa una stringa composta.
  void		printf(const char *format, ...) {
    va_list args;
    char *buf;
    
    va_start(args, format);
    buf = g_strdup_vprintf(format, args);
    va_end(args);
    
    check_special_chars(buf, 1);
    xchat_print(hPlugin, buf);
    g_free(buf);
  }

  // Genera un Evento.
  void		emit_print(const char *event_name, 
			   ...) {
    if (event_name && event_name[0] != '\0') {
      va_list args;
      char	*argv[4];
      int	i = 0;
      
      memset (&argv, 0, sizeof (argv));
      va_start(args, event_name);
      
      while(1)	{
	argv[i] = va_arg (args, char *);
	if (!argv[i])
	  break;
	i++;
	if (i >= 4)
	  break;
      }
      
      xchat_emit_print(hPlugin, event_name, argv[0], argv[1], argv[2], argv[3]);
      
      va_end (args);
    }
  }

  // Restituisce il contesto in base al server/canale.
  xchat_context	*find_context(char *servname = NULL,	// Server nel quale cercare
			      char *channel = NULL) {	// Canale da cercare
    return xchat_find_context(hPlugin, servname, channel);
  }

  // Cerca un oggetto della classe Utente in base al: Nickname
  UserData	*find_user_nickname(const char *Nickname) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    if (Nickname && Nickname[0] != '\0') {
      // Se la lista non e' gia' stata creata, provvede a farlo.
      if (!cUserData_list)
	cUserData_list = Plugin::list_get_cUserData();

      // Verifica se e' riuscito a crare la lista.
      if (cUserData_list) {
	while(Plugin::list_next_cUserData(cUserData_list)) {
	  pcUserData = Plugin::list_cUserData(cUserData_list);
	  
	  if (!strcasecmp(Nickname, pcUserData->GetNickname()))
	    return pcUserData;
	  
	  delete pcUserData;
	}
	
	Plugin::list_free_cUserData(&cUserData_list);
      }
    }
    
    return NULL;
  }

  // Cerca un oggetto della classe Utente in base al: Host
  UserData	*find_user_host(const char *Host) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    if (Host && Host[0] != '\0') {
      // Se la lista non e' gia' stata creata, provvede a farlo.
      if (!cUserData_list)
	cUserData_list = Plugin::list_get_cUserData();

      // Verifica se e' riuscito a crare la lista.
      if (cUserData_list) {
	while(Plugin::list_next_cUserData(cUserData_list)) {
	  pcUserData = Plugin::list_cUserData(cUserData_list);
	  
	  if (!strcasecmp(Host, pcUserData->GetHost()))
	    return pcUserData;
	  
	  delete pcUserData;
	}
	
	Plugin::list_free_cUserData(&cUserData_list);
      }
    }
    
    return NULL;
  }

  // Cerca un oggetto della classe Utente in base al: Prefix
  UserData	*find_user_prefix(const char *Prefix) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    if (Prefix && Prefix[0] != '\0') {
      // Se la lista non e' gia' stata creata, provvede a farlo.
      if (!cUserData_list)
	cUserData_list = Plugin::list_get_cUserData();

      // Verifica se e' riuscito a crare la lista.
      if (cUserData_list) {
	while(Plugin::list_next_cUserData(cUserData_list)) {
	  pcUserData = Plugin::list_cUserData(cUserData_list);
	  
	  if (!strcasecmp(Prefix, pcUserData->GetPrefix()))
	    return pcUserData;
	  
	  delete pcUserData;
	}
	
	Plugin::list_free_cUserData(&cUserData_list);
      }
    }
    
    return NULL;
  }

  // Cerca un oggetto della classe Utente in base al: Channel
  UserData	*find_user_channel(const char *Channel) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    if (Channel && Channel[0] != '\0') {
      // Se la lista non e' gia' stata creata, provvede a farlo.
      if (!cUserData_list)
	cUserData_list = Plugin::list_get_cUserData();

      // Verifica se e' riuscito a crare la lista.
      if (cUserData_list) {
	while(Plugin::list_next_cUserData(cUserData_list)) {
	  pcUserData = Plugin::list_cUserData(cUserData_list);
	  
	  if (!strcasecmp(Channel, pcUserData->GetChannel()))
	    return pcUserData;
	  
	  delete pcUserData;
	}
	
	Plugin::list_free_cUserData(&cUserData_list);
      }
    }
    
    return NULL;
  }

  // Cerca un oggetto della classe Utente in base al: Server
  UserData	*find_user_server(const char *Server) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    if (Server && Server[0] != '\0') {
      // Se la lista non e' gia' stata creata, provvede a farlo.
      if (!cUserData_list)
	cUserData_list = Plugin::list_get_cUserData();

      // Verifica se e' riuscito a crare la lista.
      if (cUserData_list) {
	while(Plugin::list_next_cUserData(cUserData_list)) {
	  pcUserData = Plugin::list_cUserData(cUserData_list);
	  
	  if (!strcasecmp(Server, pcUserData->GetServer()))
	    return pcUserData;
	  
	  delete pcUserData;
	}
	
	Plugin::list_free_cUserData(&cUserData_list);
      }
    }
    
    return NULL;
  }

  // Cerca un oggetto della classe Utente in base al: Contesto
  UserData	*find_user_CTX(const xchat_context *CTX) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    if (CTX) {      
      // Se la lista non e' gia' stata creata, provvede a farlo.
      if (!cUserData_list)
	cUserData_list = Plugin::list_get_cUserData();

      // Verifica se e' riuscito a crare la lista.
      if (cUserData_list) {
	while(Plugin::list_next_cUserData(cUserData_list)) {
	  pcUserData = Plugin::list_cUserData(cUserData_list);
	  
	  if (CTX == pcUserData->GetCTX())
	    return pcUserData;
	  
	  delete pcUserData;
	}
	
	Plugin::list_free_cUserData(&cUserData_list);
      }
    }
    
    return NULL;
  }

  // Cerca un oggetto della classe Utente in base al flag: Ignored
  UserData	*find_user_ignored(int Ignored) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    // Se la lista non e' gia' stata creata, provvede a farlo.
    if (!cUserData_list)
      cUserData_list = Plugin::list_get_cUserData();
    
    // Verifica se e' riuscito a crare la lista.
    if (cUserData_list) {
      while(Plugin::list_next_cUserData(cUserData_list)) {
	pcUserData = Plugin::list_cUserData(cUserData_list);
	
	if (Ignored == pcUserData->GetIgnored())
	  return pcUserData;
	
	delete pcUserData;
      }
      
      Plugin::list_free_cUserData(&cUserData_list);
    }
    
    return NULL;
  }

  // Cerca un oggetto della classe Utente in base ai dati passati come
  // parametri.
  /*
   * I parametri che vengono passati "non nulli" devono essere
   * verificati completamente (cioe' il relativo dato dell'oggetto
   * UserData deve essere identico anche se con case insensitive)
   * affinche' l'oggetto UserData venga ritornato, viceversa, vengono
   * ignorati (in questo modo e' possibile fare ricerche "incrociate"
   * solo su alcuni dati).
   *
   */
  UserData	*find_user_cmp(const char *Nickname = NULL, 
			       const char *Host = NULL, 
			       const char *Prefix = NULL, 
			       const char *Channel = NULL, 
			       const char *Server = NULL, 
			       const xchat_context *CTX = NULL, 
			       int Ignored = -1) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    // Se la lista non e' gia' stata creata, provvede a farlo.
    if (!cUserData_list)
      cUserData_list = Plugin::list_get_cUserData();
    
    // Verifica se e' riuscito a crare la lista.
    if (cUserData_list) {
      while(Plugin::list_next_cUserData(cUserData_list)) {
	pcUserData = Plugin::list_cUserData(cUserData_list);
	
	if (!Nickname || 
	    !Plugin::nickcmp(Nickname, 
			     pcUserData->GetNickname() ? 
			     pcUserData->GetNickname() : 
			     "")) {
	  if (!Host || 
	      !Plugin::nickcmp(Host, 
			       pcUserData->GetHost() ? 
			       pcUserData->GetHost() : 
			       "")) {
	    if (!Prefix || 
		!Plugin::nickcmp(Prefix, 
				 pcUserData->GetPrefix() ? 
				 pcUserData->GetPrefix() : 
				 "")) {
	      if (!Channel || 
		  !Plugin::nickcmp(Channel, 
				   pcUserData->GetChannel() ? 
				   pcUserData->GetChannel() : 
				   "")) {
		if (!Server || 
		    !Plugin::nickcmp(Server, 
				     pcUserData->GetServer() ? 
				     pcUserData->GetServer() : 
				     "")) {
		  if (!CTX || 
		      CTX == pcUserData->GetCTX()) {
		    if (Ignored == -1 || 
			Ignored == pcUserData->GetIgnored())
		      return pcUserData;
		  }
		}
	      }
	    }
	  }
	}
	
	delete pcUserData;
      }
      
      Plugin::list_free_cUserData(&cUserData_list);
    }
    
    return NULL;
  }

  // Cerca un oggetto della classe Utente in base ai dati passati come
  // parametri.
  /*
   * I parametri che vengono passati "non nulli" possono essere
   * verificati anche parzialmente (cioe' il relativo dato
   * dell'oggetto UserData deve essere simile anche se con case
   * sensitive) affinche' l'oggetto UserData venga ritornato,
   * viceversa, vengono ignorati (in questo modo e' possibile fare
   * ricerche "incrociate" solo su alcuni dati).
   *
   */
  UserData	*find_user_str(const char *Nickname = NULL, 
			       const char *Host = NULL, 
			       const char *Prefix = NULL, 
			       const char *Channel = NULL, 
			       const char *Server = NULL, 
			       const xchat_context *CTX = NULL, 
			       int Ignored = -1) {
    static xchat_list	*cUserData_list;
    UserData		*pcUserData = NULL;

    // Se la lista non e' gia' stata creata, provvede a farlo.
    if (!cUserData_list)
      cUserData_list = Plugin::list_get_cUserData();
    
    // Verifica se e' riuscito a crare la lista.
    if (cUserData_list) {
      while(Plugin::list_next_cUserData(cUserData_list)) {
	pcUserData = Plugin::list_cUserData(cUserData_list);
	
	if (!Nickname || 
	    strstr(Nickname, 
		   pcUserData->GetNickname() ? 
		   pcUserData->GetNickname() : 
		   " ") || 
	    strstr(pcUserData->GetNickname() ? 
		   pcUserData->GetNickname() : 
		   " ", 
		   Nickname)) {
	  if (!Host || 
	      strstr(Host, 
		     pcUserData->GetHost() ? 
		     pcUserData->GetHost() : 
		     " ") || 
	      strstr(pcUserData->GetHost() ? 
		     pcUserData->GetHost() : 
		     " ", 
		     Host)) {
	    if (!Prefix || 
		strstr(Prefix, 
		       pcUserData->GetPrefix() ? 
		       pcUserData->GetPrefix() : 
		       " ") || 
		strstr(pcUserData->GetPrefix() ? 
		       pcUserData->GetPrefix() : 
		       " ", 
		       Prefix)) {
	      if (!Channel || 
		  strstr(Channel, 
			 pcUserData->GetChannel() ? 
			 pcUserData->GetChannel() : 
			 " ") || 
		  strstr(pcUserData->GetChannel() ? 
			 pcUserData->GetChannel() : 
			 " ", 
			 Channel)) {
		if (!Server || 
		    strstr(Server, 
			   pcUserData->GetServer() ? 
			   pcUserData->GetServer() : 
			   " ") || 
		    strstr(pcUserData->GetServer() ? 
			   pcUserData->GetServer() : 
			   " ", 
			   Server)) {
		  if (!CTX || 
		      CTX == pcUserData->GetCTX()) {
		    if (Ignored == -1 || 
			Ignored == pcUserData->GetIgnored())
		      return pcUserData;
		  }
		}
	      }
	    }
	  }
	}
	
	delete pcUserData;
      }
      
      Plugin::list_free_cUserData(&cUserData_list);
    }
    
    return NULL;
  }

  // Restituisce il contesto corrente.
  xchat_context	*get_context() {
    return xchat_get_context(hPlugin);
  }

  // Ritorna informazioni sul contesto corrente.
  const char	*get_info(char *id) {			// Stringa identificativa
    if (id && id[0] != '\0') {
      return xchat_get_info(hPlugin, id);
    }

    return NULL;
  }

  // Ritorna informazioni sullo stato di AWAY dell'utente locale,
  // compreso l'eventuale motivo.
  const char	*get_info_away() {
    return xchat_get_info(hPlugin, "away");
  }

  // Ritorna il nome del canale corrente.
  const char	*get_info_channel() {
    return xchat_get_info(hPlugin, "channel");
  }

  // Ritorna il nome dell'Host corrente.
  const char	*get_info_host() {
    return xchat_get_info(hPlugin, "host");
  }

  // Ritorna il nome del network corrente.
  const char	*get_info_network() {
    return xchat_get_info(hPlugin, "network");
  }

  // Ritorna il nickname corrente dell'utente locale.
  const char	*get_info_nick() {
    return xchat_get_info(hPlugin, "nick");
  }

  // Ritorna il nome del server corrente.
  const char	*get_info_server() {
    return xchat_get_info(hPlugin, "server");
  }

  // Ritorna il topic del canale corrente.
  const char	*get_info_topic() {
    return xchat_get_info(hPlugin, "topic");
  }

  // Ritorna la versione di XChat.
  const char	*get_info_version() {
    return xchat_get_info(hPlugin, "version");
  }

  // Ritorna la directory di configurazione di XChat.
  const char	*get_info_xchatdir() {
    return xchat_get_info(hPlugin, "xchatdir");
  }

  // Restituisce la configurazione di una variabile di XChat.
  // 0-Fallimento 1-Ritorna una stringa 2-Ritorna un Intero
  // 3-Ritorna un Booleano.
  int		get_prefs(const char *name,	// Nome della preferenza
			  const char **string,	// &stringa
			  int *integer) {	// &intero
    if (name && name[0] != '\0')
      return xchat_get_prefs(hPlugin, name, string, integer);
    else
      return 0;
  }

  // Imposta come corrente il contesto passato.
  int		set_context(xchat_context *ctx) {
    if (ctx)
      return xchat_set_context(hPlugin, ctx);
    
    return 0;
  }

  // Ottiene le informazioni richieste inserendole in una lista.
  /*
   * I possibili valori di name sono:
   *
   * channels
   * dcc
   * ignore
   * users
   *
   */
  /*
   *  Se, come secondo parametro, viene passata la define
   *  "DONT_FREE_PREVIOUS_LIST", la lista precedente non viene
   *  eliminata.
   *
   */
  xchat_list	*list_get(const char *name, 
			  int fFreePreviousList = 1) {
    // Se necessario, libera la lista corrente.
    if (name && name[0] != '\0') {
      if (hlData && fFreePreviousList) {
	xchat_list_free(hPlugin, hlData);
	hlData = NULL;
      }
      
      hlData = xchat_list_get(hPlugin, name);
      eListStatus = list_OK;
      
      if (!hlData || 
	  !Plugin::list_next(hlData)) {
	eListStatus = list_ERROR;
	
	return NULL;
      }
      
      return hlData;
    }
    
    eListStatus = list_ERROR;
    
    return 0;
  }

  // Ottiene le informazioni sui canali.
  xchat_list	*list_get_channels(int fFreePreviousList = 1) {
    return Plugin::list_get("channels", fFreePreviousList);
  }

  // Ottiene le informazioni sui dcc.
  xchat_list	*list_get_dcc(int fFreePreviousList = 1) {
    return Plugin::list_get("dcc", fFreePreviousList);
  }

  // Ottiene le informazioni sugli ignore.
  xchat_list	*list_get_ignore(int fFreePreviousList = 1) {
    return Plugin::list_get("ignore", fFreePreviousList);
  }

  // Ottiene le informazioni sugli utenti.
  xchat_list	*list_get_users(int fFreePreviousList = 1) {
    return Plugin::list_get("users", fFreePreviousList);
  }

  // Genera la lista utenti sfruttando la classe UserData.
  xchat_list	*list_get_cUserData() {
    // Dichiara le liste
    xchat_list		*list = NULL;
    GSList		*cUserData_list = NULL;
    xchat_list		*user_list = NULL;
    xchat_list		*ctx_list = NULL;
    xchat_context	*ctx = NULL;
    xchat_list		*ignore_list = NULL;
    UserData		*pcUserDataInsert = NULL;
    int			bIgnoreListEmpty = 0;
    
    // Memorizza il contesto corrente.
    xchat_context	*ctxCurrent = xchat_get_context(hPlugin);
    
    list = (xchat_list*)malloc(sizeof(struct _xchat_list));
    if (list) {
      list->pos = NULL;
      list->type = LIST_USERS;
      
      // Esegue un ciclo iterativo per ogni contesto esistente.
      ctx_list = xchat_list_get(hPlugin, "channels");
      if (ctx_list) {
	while(xchat_list_next(hPlugin, ctx_list)) {
	  ctx = (xchat_context*)xchat_list_str(hPlugin, ctx_list, "context");
	  
	  // Se il contesto intercettato e' valido, provvede ad
	  // impostarlo come corrente.
	  if (ctx && xchat_set_context(hPlugin, ctx)) {
	    // Se il contesto individuato e' un Canale, provvede ad
	    // inserire tutti gli utenti presenti in quest'ultimo. Fa
	    // la stessa cosa anche se si tratta di un
	    // Dialog. L'inserimento avviene solo se l'utente non e'
	    // gia' inserito.
	    user_list = xchat_list_get(hPlugin, "users");
	    
	    if (user_list) {
	      while(xchat_list_next(hPlugin, user_list)) {
		// Crea un utente da inserire nella GSList.
		pcUserDataInsert = new UserData(xchat_list_str(hPlugin, user_list, "nick"), 
						xchat_list_str(hPlugin, user_list, "host"), 
						xchat_list_str(hPlugin, user_list, "prefix"), 
						xchat_list_str(hPlugin, ctx_list, "channel"), 
						xchat_list_str(hPlugin, ctx_list, "server"), 
						ctx);

		if (pcUserDataInsert) {
		  // Se la lista degli ignore non e' risultata vuota in
		  // precedenza, provvede ad effettuare il controllo.
		  if (!bIgnoreListEmpty) {
		    // Ora deve verificare se l'utente estrapolato
		    // corrisponde ad una delle eventuali maschere di
		    // ignore; se combacia, imposta il relativo flag.
		    ignore_list = xchat_list_get(hPlugin, "ignore");
		    
		    if (ignore_list) {
		      while(xchat_list_next(hPlugin, ignore_list) && 
			    !pcUserDataInsert->
			    SetIgnored(xchat_list_str(hPlugin, 
						      ignore_list, 
						      "mask"))) {}
		      
		      // Ora puo' liberare la lista, non piu'
		      // indispensabile.
		      xchat_list_free(hPlugin, ignore_list); ignore_list = NULL;
		    } else
		      bIgnoreListEmpty = 1;
		  }
		  
		  // Se la lista degli utenti e' vuota, inserisce l'utente
		  // creato.
		  if (!cUserData_list) {
		    cUserData_list = g_slist_alloc();
		    cUserData_list->data = (void*)pcUserDataInsert;
		  } else {
		    // Appende l'utente nella lista.
		    if (!g_slist_append(cUserData_list, 
					pcUserDataInsert)) {		      
		      // Se non riesce a creare la lista, ritorna
		      // errore.
		      free(list); list = NULL;
		      eListStatus = list_ERROR;
		      return list;
		    }
		  }
		}
	      }
	      
	      // Ora puo' liberare la lista, non piu'
	      // indispensabile.
	      xchat_list_free(hPlugin, user_list); user_list = NULL;
	    }
	  }
	}
	
	// Ora puo' liberare la lista, non piu'
	// indispensabile.
	xchat_list_free(hPlugin, ctx_list); ctx_list = NULL;
      }

      xchat_set_context(hPlugin, ctxCurrent);
      
      if (cUserData_list) {
	// ritorna la lista creata.
	list->head = list->next = cUserData_list;
	
	eListStatus = list_OK;
      } else {
	free(list); list = NULL;
	eListStatus = list_ERROR;
      }
    }	// list: ok
    
    return list;
  }

  // Elimina la lista utenti creata sfruttando la classe UserData.
  void		list_free_cUserData(xchat_list **xlist) {
    if ((*xlist)) {
      xchat_list_free(hPlugin, (*xlist));
      (*xlist) = NULL;
    }
  }

  // Ritorna gli elementi della lista formata da elementi di classe
  // UserData.
  UserData	*list_cUserData(xchat_list *xlist) {
    if (xlist && xlist->pos)
      return (UserData*)xlist->pos->data;
    
    eListStatus = list_ERROR;
    
    // Evita di ritornare il nulla.
    UserData	*pUser = new UserData();
    return pUser;
  }

  // Punta all'elemento successivo della lista formata da elementi di
  // classe UserData.
  int		list_next_cUserData(xchat_list *xlist) {
    if (xlist) {
      if (xlist->next == NULL)
	return 0;
      
      xlist->pos = xlist->next;
      xlist->next = xlist->pos->next;
      
      return 1;
    }
    
    return 0;
  }
  
  // Elimina la lista passata come parametro, oppure, quella globale
  // se la prima risulta nulla.
  void		list_free(xchat_list **xlist) {
    if ((*xlist) == hlData) {
      hlData = NULL;
    }
    
    if ((*xlist)) {
      xchat_list_free(hPlugin, (*xlist));
      (*xlist) = NULL;
    } else
      Plugin::list_free();
  }

  // Elimina la lista.
  void		list_free() {
    if (hlData) {
      xchat_list_free(hPlugin, hlData);
      hlData = NULL;
    }
  }

  // Punta all'elemento successivo della lista.
  int		list_next(xchat_list *xlist = NULL) {
    if (xlist == NULL)
      xlist = hlData;

    if (xlist && 
	xchat_list_next(hPlugin, xlist))
      return 1;

    return 0;
  }

  // Ritorna la tipologia dei campi.
  /*
   * I possibili valori di name sono:
   *
   * channels
   * dcc
   * ignore
   * users
   * lists
   *
   */
  const char* const*list_fields(const char *name) {
    if (name && name[0] != '\0')
      return xchat_list_fields(hPlugin, name);

    return NULL;
  }

  // Ritorna la stringa appartenente al campo specificato.
  /*
   * I possibili valori di name sono:
   *
   * channel
   * context (meglio usare list_channels_context_ctx() )
   * server
   * destfile
   * file
   * nick
   * mask
   * host
   * prefix
   *
   */
  const char	*list_str(const char *name, 
			  xchat_list *xlist = NULL) {
    if (xlist == NULL)
      xlist = hlData;

    if (xlist && 
	name && 
	name[0] != '\0') {
      return xchat_list_str(hPlugin, xlist, name);
    }

    return NULL;
  }

  // Ritorna la lista dei canali "joinati".
  const char	*list_channels_channel_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("channels");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "channel");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna il puntatore ai contesti "joinati".
  const xchat_context*
    list_channels_context_ctx(xchat_list *xlist = NULL) {
    xchat_context	*pContext = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("channels");
    
    if (eListStatus) {
      pContext = (xchat_context*)xchat_list_str(hPlugin, xlist, "context");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pContext;
  }

  // Ritorna la lista dei servers collegati.
  const char	*list_channels_server_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("channels");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "server");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna la lista dei nomi dei files destinazione nelle
  // trasmissioni DCC.
  const char	*list_dcc_destfile_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "destfile");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna la lista dei nomi dei files spediti/ricevuti nelle
  // trasmissioni DCC.
  const char	*list_dcc_file_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "file");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna la lista dei nick delle trasmissioni DCC.
  const char	*list_dcc_nick_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "nick");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna la lista delle maschere degli utenti ignorati.
  const char	*list_ignore_mask_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("ignore");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "mask");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna la lista dei nick degli utenti contenuti nel canale
  // corrente.
  const char	*list_users_nick_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("users");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "nick");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna la lista degli host degli utenti contenuti nel canale
  // corrente.
  const char	*list_users_host_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("users");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "host");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna la lista dei prefissi (@, %, +) eventualmente associati
  // agli utenti presenti nel canale corrente.
  const char	*list_users_prefix_str(xchat_list *xlist = NULL) {
    const char	*pData = NULL;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("users");
    
    if (eListStatus) {
      pData = xchat_list_str(hPlugin, xlist, "prefix");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return pData;
  }

  // Ritorna il numero appartenente al campo specificato.
  /*
   * I possibili valori di name sono:
   *
   * type
   * address32
   * cps
   * port
   * pos
   * resume
   * size
   * status
   * type
   * flags
   *
   */
  int		list_int(const char *name, 
			 xchat_list *xlist = NULL) {
    if (xlist == NULL)
      xlist = hlData;

    if (xlist && 
	name && 
	name[0] != '\0') {
      return xchat_list_int(hPlugin, xlist, name);
    }
    
    return 0;
  }

  // Ritorna la lista delle tipologie delle sessioni create.
  int		list_channels_type_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("channel");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "type");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista degli indirizzi ipv4 delle trasmissioni DCC.
  int		list_dcc_address32_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "address32");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista delle velocita' delle trasmissioni DCC.
  int		list_dcc_cps_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "cps");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista delle porte usate nelle trasmissioni DCC.
  int		list_dcc_port_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "port");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista delle posizioni di resume delle trasmissioni
  // DCC.
  int		list_dcc_pos_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "pos");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista dei resume delle trasmissioni DCC.
  int		list_dcc_resume_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "resume");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista delle dimensioni delle trasmissioni DCC.
  int		list_dcc_sizehigh_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "sizehigh");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista delle dimensioni delle trasmissioni DCC.
  unsigned int		list_dcc_size_int(xchat_list *xlist = NULL) {
    unsigned int	uiData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      uiData = (unsigned)xchat_list_int(hPlugin, xlist, "size");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return uiData;
  }

  // Ritorna la lista dello stato delle trasmissioni DCC.
  int		list_dcc_status_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "status");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista delle tipologie delle trasmissioni DCC.
  int		list_dcc_type_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("dcc");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "type");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Ritorna la lista dei flags relativa agli ignore attivi.
  int		list_ignore_flags_int(xchat_list *xlist = NULL) {
    int		iData = 0;

    if (xlist == NULL)
      xlist = hlData;
    
    if (!xlist)
      xlist = Plugin::list_get("ignore");
    
    if (eListStatus) {
      iData = xchat_list_int(hPlugin, xlist, "flags");
      
      if (!Plugin::list_next(xlist))    
	eListStatus = list_ERROR;
    } else {
      Plugin::list_free(&xlist);
    }
    
    return iData;
  }

  // Esegue un confronto tra due stringhe (nickname/canale) seguendo
  // le direttive della RFC1459.
  int		nickcmp(const char *s1, const char *s2) {
    int		result = -2;
    char	*s1cpy, *s2cpy;

    if (s1 && s2 && 
	s1[0] != '\0' && s2[0] != '\0') {
      s1cpy = strdup(s1);
      s2cpy = strdup(s2);
      result = xchat_nickcmp(hPlugin, s1cpy, s2cpy);
      free(s1cpy);
      free(s2cpy);
    }
    
    return result;
  }
};

// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
#endif
// //////////////////////////////////////////////////////////////////////////////////////////////////////////
//___________________________________________________________________________________________________________
// End $Source: /home/uda/src/c/buduscript/RCS/plugin.h,v $
